@@ -163,7 +163,7 @@ struct kfd_ioctl_set_memory_policy_args {
 	__u32 gpu_id;			/* to KFD */
 	__u32 default_policy;		/* to KFD */
 	__u32 alternate_policy;		/* to KFD */
-	__u32 pad;
+	__u32 misc_process_flag;        /* to KFD */
 };
 
 /*
@@ -363,6 +363,9 @@ enum kfd_dbg_trap_exception_code {
 #define KFD_DBG_EC_TYPE_IS_PROCESS(ecode)				\
 			(!!(KFD_EC_MASK(ecode) & KFD_EC_MASK_PROCESS))
 
+/* Misc. per process flags */
+#define ENABLE_MFMA_HIGH_PRECISION              (1 << 0)
+
 enum kfd_dbg_runtime_state {
 	DEBUG_RUNTIME_STATE_DISABLED = 0,
 	DEBUG_RUNTIME_STATE_ENABLED = 1,
@@ -1188,9 +1191,8 @@ enum kfd_ioctl_spm_op {
  * @buf_size[in]:      size of the destination buffer
  * @timeout[in/out]:   [in]: timeout in milliseconds, [out]: amount of time left
  *                      `in the timeout window
- * @bytes_copied[out]: amount of data that was copied to the previous dest_buf
- * @has_data_loss:     boolean indicating whether data was lost
- *                      (e.g. due to a ring-buffer overflow)
+ * @bytes_copied[out]: total amount of data that was copied to the previous dest_buf
+ * @has_data_loss:     total count for sub-block which has data loss
  *
  * This ioctl performs different functions depending on the @op parameter.
  *
@@ -1240,6 +1242,21 @@ struct kfd_ioctl_spm_args {
 	__u32 has_data_loss;
 };
 
+/**
+ * kfd_ioctl_spm_buffer_header - SPM Buffer header for kfd_ioctl_spm_args->dest_buf
+ *
+ * @version        [out]: spm versiom
+ * @bytes_copied   [out]: amount of data for each sub-block
+ * @has_data_loss: [out]: boolean indicating whether data was lost for each sub-block
+ *                        (e.g. due to a ring-buffer overflow)
+ */
+struct kfd_ioctl_spm_buffer_header {
+	__u32 version; /* 0-23: minor 24-31: major */
+	__u32 bytes_copied;
+	__u32 has_data_loss;
+	__u32 reserved[5];
+};
+
 /**************************************************************************************************
  * CRIU IOCTLs (Checkpoint Restore In Userspace)
  *
