@@ -2,24 +2,24 @@
 //
 // The University of Illinois/NCSA
 // Open Source License (NCSA)
-// 
-// Copyright (c) 2014-2020, Advanced Micro Devices, Inc. All rights reserved.
-// 
+//
+// Copyright (c) 2014-2025, Advanced Micro Devices, Inc. All rights reserved.
+//
 // Developed by:
-// 
+//
 //                 AMD Research and AMD HSA Software Development
-// 
+//
 //                 Advanced Micro Devices, Inc.
-// 
+//
 //                 www.amd.com
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
 // deal with the Software without restriction, including without limitation
 // the rights to use, copy, modify, merge, publish, distribute, sublicense,
 // and/or sell copies of the Software, and to permit persons to whom the
 // Software is furnished to do so, subject to the following conditions:
-// 
+//
 //  - Redistributions of source code must retain the above copyright notice,
 //    this list of conditions and the following disclaimers.
 //  - Redistributions in binary form must reproduce the above copyright
@@ -29,7 +29,7 @@
 //    nor the names of its contributors may be used to endorse or promote
 //    products derived from this Software without specific prior written
 //    permission.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
@@ -55,7 +55,9 @@
 namespace rocr {
 namespace AMD {
 CpuAgent::CpuAgent(HSAuint32 node, const HsaNodeProperties &node_props)
-    : core::Agent(core::DriverType::KFD, node, kAmdCpuDevice),
+    : core::Agent(
+          core::Runtime::runtime_singleton_->AgentDriver(core::DriverType::KFD),
+          node, kAmdCpuDevice),
       properties_(node_props) {
   InitRegionList();
 
@@ -71,8 +73,7 @@ void CpuAgent::InitRegionList() {
   const bool is_apu_node = (properties_.NumFComputeCores > 0);
 
   std::vector<HsaMemoryProperties> mem_props(properties_.NumMemoryBanks);
-  if (HSAKMT_STATUS_SUCCESS ==
-      hsaKmtGetNodeMemoryProperties(node_id(), properties_.NumMemoryBanks, &mem_props[0])) {
+  if (HSA_STATUS_SUCCESS == driver().GetMemoryProperties(node_id(), mem_props)) {
     std::vector<HsaMemoryProperties>::iterator system_prop =
         std::find_if(mem_props.begin(), mem_props.end(), [](HsaMemoryProperties prop) -> bool {
           return (prop.SizeInBytes > 0 && prop.HeapType == HSA_HEAPTYPE_SYSTEM);
@@ -105,9 +106,8 @@ void CpuAgent::InitRegionList() {
 void CpuAgent::InitCacheList() {
   // Get CPU cache information.
   cache_props_.resize(properties_.NumCaches);
-  if (HSAKMT_STATUS_SUCCESS !=
-      hsaKmtGetNodeCacheProperties(node_id(), properties_.CComputeIdLo,
-                                   properties_.NumCaches, &cache_props_[0])) {
+  if (HSA_STATUS_SUCCESS !=
+      driver().GetCacheProperties(node_id(), properties_.CComputeIdLo, cache_props_)) {
     cache_props_.clear();
   } else {
     // Only store CPU D-cache.
@@ -181,13 +181,24 @@ hsa_status_t CpuAgent::IterateCache(hsa_status_t (*callback)(hsa_cache_t cache,
   return HSA_STATUS_SUCCESS;
 }
 
+hsa_status_t CpuAgent::IterateSupportedIsas(
+                  hsa_status_t (*callback)(hsa_isa_t isa, void* data),
+                                                          void* data) const {
+  AMD::callback_t<decltype(callback)> call(callback);
+  for (const auto& isa : supported_isas()) {
+    hsa_status_t stat = call(core::Isa::Handle(isa), data);
+    if (stat != HSA_STATUS_SUCCESS) return stat;
+  }
+  return HSA_STATUS_SUCCESS;
+}
+
 hsa_status_t CpuAgent::GetInfo(hsa_agent_info_t attribute, void* value) const {
-  
+
   // agent, and vendor name size limit
   const size_t attribute_u = static_cast<size_t>(attribute);
-  
+
   switch (attribute_u) {
-    
+
     // The code copies HsaNodeProperties.MarketingName a Unicode string
     // which is encoded in UTF-16 as a 7-bit ASCII string. The value of
     // HsaNodeProperties.MarketingName is obtained from the "model name"
@@ -402,6 +413,13 @@ hsa_status_t CpuAgent::GetInfo(hsa_agent_info_t attribute, void* value) const {
     case HSA_AMD_AGENT_INFO_AQL_EXTENSIONS:
       memset(value, 0, sizeof(uint8_t) * 8);
       break;
+    case HSA_AMD_AGENT_INFO_SCRATCH_LIMIT_MAX:
+    case HSA_AMD_AGENT_INFO_SCRATCH_LIMIT_CURRENT:
+      *((uint64_t*)value) = 0;
+      break;
+    case HSA_AMD_AGENT_INFO_CLOCK_COUNTERS:
+      memset(value, 0, sizeof(hsa_amd_clock_counters_t));
+      break;
     default:
       return HSA_STATUS_ERROR_INVALID_ARGUMENT;
       break;
@@ -409,10 +427,9 @@ hsa_status_t CpuAgent::GetInfo(hsa_agent_info_t attribute, void* value) const {
   return HSA_STATUS_SUCCESS;
 }
 
-hsa_status_t CpuAgent::QueueCreate(size_t size, hsa_queue_type32_t queue_type,
-                                   core::HsaEventCallback event_callback,
-                                   void* data, uint32_t private_segment_size,
-                                   uint32_t group_segment_size,
+hsa_status_t CpuAgent::QueueCreate(size_t size, hsa_queue_type32_t queue_type, uint64_t flags,
+                                   core::HsaEventCallback event_callback, void* data,
+                                   uint32_t private_segment_size, uint32_t group_segment_size,
                                    core::Queue** queue) {
   // No HW AQL packet processor on CPU device.
   return HSA_STATUS_ERROR;
