@@ -83,7 +83,6 @@ void* __stdcall ThreadTrampoline(void* arg) {
   ThreadArgs* ar = (ThreadArgs*)arg;
   ThreadEntry CallMe = ar->entry_function;
   void* Data = ar->entry_args;
-  delete ar;
   CallMe(Data);
   return nullptr;
 }
@@ -91,15 +90,17 @@ void* __stdcall ThreadTrampoline(void* arg) {
 // Thread container allows multiple waits and separate close (destroy).
 class os_thread {
  public:
-  explicit os_thread(ThreadEntry function, void* threadArgument, uint stackSize)
+  explicit os_thread(ThreadEntry function,
+                      void* threadArgument,
+                      uint stackSize,
+                      int priority)
       : thread(0), lock(nullptr), state(RUNNING) {
     int err;
-    std::unique_ptr<ThreadArgs> args(new ThreadArgs);
     lock = CreateMutex();
     if (lock == nullptr) return;
 
-    args->entry_args = threadArgument;
-    args->entry_function = function;
+    args.entry_args = threadArgument;
+    args.entry_function = function;
 
     pthread_attr_t attrib;
     err = pthread_attr_init(&attrib);
@@ -137,16 +138,18 @@ class os_thread {
       for (int i = 0; i < cores; i++) {
         CPU_SET_S(i, CPU_ALLOC_SIZE(cores), cpuset);
       }
+#ifdef HAVE_PTHREAD_ATTR_SETAFFINITY_NP
       err = pthread_attr_setaffinity_np(&attrib, CPU_ALLOC_SIZE(cores), cpuset);
       CPU_FREE(cpuset);
       if (err != 0) {
         fprintf(stderr, "pthread_setaffinity_np failed: %s\n", strerror(err));
         return;
       }
+#endif
     }
 
     do {
-      err = pthread_create(&thread, &attrib, ThreadTrampoline, args.get());
+      err = pthread_create(&thread, &attrib, ThreadTrampoline, &args);
       if (!err) break;
 
       if (err != EINVAL || stackSize == 0) {
@@ -165,11 +168,53 @@ class os_thread {
       }
     } while (stackSize < 20 * 1024 * 1024);
 
-    args.release();
+#ifndef HAVE_PTHREAD_ATTR_SETAFFINITY_NP
+    if (cores && cpuset) {
+      err = pthread_setaffinity_np(thread, CPU_ALLOC_SIZE(cores), cpuset);
+      CPU_FREE(cpuset);
+      if (err != 0) {
+        fprintf(stderr, "pthread_setaffinity_np failed: %s\n", strerror(err));
+        thread = 0;
+        return;
+      }
+    }
+#endif
+    struct sched_param param = {};
+    if (priority != OS_THREAD_PRIORITY_DEFAULT) {
+      int set_priority;
+      int max_priority = sched_get_priority_max(SCHED_FIFO);
+
+      if (priority == OS_THREAD_PRIORITY_MAX)
+        set_priority = max_priority;
+      else if (priority == OS_THREAD_PRIORITY_HIGH)
+        set_priority = max_priority - 1;
+      else if (priority > max_priority)
+        set_priority = max_priority;
+      else
+        set_priority = priority;
+
+      param.sched_priority = set_priority;
+      if (pthread_setschedparam(thread, SCHED_FIFO, &param)) {
+        fprintf(stderr, "pthread_setschedparam failed\n");
+        return;
+      }
+
+      int policy = 0;
+      if (pthread_getschedparam(thread, &policy, &param))
+        fprintf(stderr, "pthread_getschedparam failed: %s\n", strerror(err));
+
+      if (policy != SCHED_FIFO || param.sched_priority != set_priority)
+        fprintf(stderr, "Failed to adjust thread priority (policy:%s requested:%d current:%d)\n",
+                          policy == SCHED_FIFO ? "FIFO" :
+                          policy == SCHED_OTHER ? "OTHER" :
+                          policy == SCHED_RR ? "RR" : "Unknown",
+                          set_priority, param.sched_priority);
+    }
   }
 
   os_thread(os_thread&& rhs) {
     thread = rhs.thread;
+    args = rhs.args;
     lock = rhs.lock;
     state = int(rhs.state);
     rhs.thread = 0;
@@ -204,6 +249,7 @@ class os_thread {
 
  private:
   pthread_t thread;
+  struct ThreadArgs args;
   Mutex lock;
   std::atomic<int> state;
   enum { FINISHED = 0, RUNNING = 1 };
@@ -415,8 +461,8 @@ void uSleep(int delayInUs) { usleep(delayInUs); }
 
 void YieldThread() { sched_yield(); }
 
-Thread CreateThread(ThreadEntry function, void* threadArgument, uint stackSize) {
-  os_thread* result = new os_thread(function, threadArgument, stackSize);
+Thread CreateThread(ThreadEntry function, void* threadArgument, uint stackSize, int priority) {
+  os_thread* result = new os_thread(function, threadArgument, stackSize, priority);
   if (!result->Valid()) {
     delete result;
     return nullptr;
@@ -528,10 +574,12 @@ int WaitForOsEvent(EventHandle event, unsigned int milli_seconds) {
       // Timeout
       return 1;
     }
+  } else {
+      pthread_mutex_lock(&eventDescrp->mutex);
   }
 
   int ret_code = 0;
-  pthread_mutex_lock(&eventDescrp->mutex);
+  
   if (!eventDescrp->state) {
     if (milli_seconds == 0) {
       ret_code = 1;
@@ -663,29 +711,23 @@ SharedMutex CreateSharedMutex() {
     return nullptr;
   }
 
-#ifdef __GLIBC__
+#ifdef HAVE_PTHREAD_RWLOCKATTR_SETKIND_NP
   err = pthread_rwlockattr_setkind_np(&attrib, PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
   if (err != 0) {
     fprintf(stderr, "Set rw lock attribute failure: %s\n", strerror(err));
     return nullptr;
   }
-#else
-  err = pthread_rwlockattr_setkind(&attrib, PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
-  if (err != 0) {
-    fprintf(stderr, "Set rw lock attribute failure: %s\n", strerror(err));
-    return nullptr;
-  }
 #endif
 
-  pthread_rwlock_t* lock = new pthread_rwlock_t;
-  err = pthread_rwlock_init(lock, &attrib);
+  std::unique_ptr<pthread_rwlock_t> lock(new pthread_rwlock_t);
+  err = pthread_rwlock_init(lock.get(), &attrib);
   if (err != 0) {
     fprintf(stderr, "rw lock init failed: %s\n", strerror(err));
     return nullptr;
   }
 
   pthread_rwlockattr_destroy(&attrib);
-  return lock;
+  return lock.release();
 }
 
 bool TryAcquireSharedMutex(SharedMutex lock) {
