@@ -48,6 +48,7 @@
 #include <algorithm>
 #include <climits>
 
+#include "core/inc/runtime.h"
 #include "inc/hsa_ext_amd.h"
 #include "core/inc/hsa_internal.h"
 #include "addrlib/src/core/addrlib.h"
@@ -572,7 +573,7 @@ hsa_status_t ImageManagerGfx11::ModifyImageSrd(
 }
 
 hsa_status_t ImageManagerGfx11::PopulateSamplerSrd(Sampler& sampler) const {
-  const hsa_ext_sampler_descriptor_t sampler_descriptor = sampler.desc;
+  const hsa_ext_sampler_descriptor_v2_t &sampler_descriptor = sampler.desc;
 
   SQ_IMG_SAMP_WORD0 word0;
   SQ_IMG_SAMP_WORD1 word1;
@@ -580,25 +581,9 @@ hsa_status_t ImageManagerGfx11::PopulateSamplerSrd(Sampler& sampler) const {
   SQ_IMG_SAMP_WORD3 word3;
 
   word0.u32All = 0;
-  switch (sampler_descriptor.address_mode) {
-    case HSA_EXT_SAMPLER_ADDRESSING_MODE_CLAMP_TO_EDGE:
-      word0.bits.CLAMP_X = static_cast<int>(SQ_TEX_CLAMP_LAST_TEXEL);
-      break;
-    case HSA_EXT_SAMPLER_ADDRESSING_MODE_CLAMP_TO_BORDER:
-      word0.bits.CLAMP_X = static_cast<int>(SQ_TEX_CLAMP_BORDER);
-      break;
-    case HSA_EXT_SAMPLER_ADDRESSING_MODE_MIRRORED_REPEAT:
-      word0.bits.CLAMP_X = static_cast<int>(SQ_TEX_MIRROR);
-      break;
-    case HSA_EXT_SAMPLER_ADDRESSING_MODE_UNDEFINED:
-    case HSA_EXT_SAMPLER_ADDRESSING_MODE_REPEAT:
-      word0.bits.CLAMP_X = static_cast<int>(SQ_TEX_WRAP);
-      break;
-    default:
-      return HSA_STATUS_ERROR_INVALID_ARGUMENT;
-  }
-  word0.bits.CLAMP_Y = word0.bits.CLAMP_X;
-  word0.bits.CLAMP_Z = word0.bits.CLAMP_X;
+  hsa_status_t status = convertAddressMode<SQ_IMG_SAMP_WORD0, SQ_TEX_CLAMP>
+                                       (word0, sampler_descriptor.address_modes);
+  if (status != HSA_STATUS_SUCCESS) return status;
   word0.bits.FORCE_UNNORMALIZED = (sampler_descriptor.coordinate_mode ==
                                   HSA_EXT_SAMPLER_COORDINATE_MODE_UNNORMALIZED);
 
@@ -674,8 +659,34 @@ uint32_t ImageManagerGfx11::GetAddrlibSurfaceInfoNv(
       break;
 
     case HSA_EXT_IMAGE_GEOMETRY_3D:
-      in.resourceType = ADDR_RSRC_TEX_3D;
-      break;
+      {
+	      in.resourceType = ADDR_RSRC_TEX_3D;
+	      /*
+	       * 3D swizzle modes enforce alignment
+	       * of the number of slices  to the block depth.
+	       * If numSlices = 3 then the 3 slices are
+	       * interleaved for 3D locality among the 8 slices
+	       * that make up each block. This causes the memory
+	       * footprint to jump to a 3x size of the ideal size
+	       *
+	       * 'enable3DSwizzleMode' flag tests for env variable
+	       * HSA_IMAGE_ENABLE_3D_SWIZZLE_DEBUG to enable or disable
+	       * 3D swizzle:
+	       * true: Keep view3dAs2dArray = 0 for real 3D interleaving.
+	       * false: Use view3dAs2dArray = 1 to avoid the alignment
+	       *       expansion.
+	       * 2D swizzle modes can lower size overhead but may yield
+	       * suboptimal cache behavior for fully 3D volumetric
+	       * operations.
+	       */
+	      bool enable3DSwizzleMode = core::Runtime::runtime_singleton_->flag().enable_3d_swizzle();
+	      if (enable3DSwizzleMode)
+		      in.flags.view3dAs2dArray = 0;
+	      else
+		      in.flags.view3dAs2dArray = 1;
+
+	      break;
+      }
   }
   in.flags.texture = 1;
 
